import os

from P_EO import Default
from P_EO.common.time_ import get_time


class PEOBaseError(Exception):
    """
    异常基类
    """

    def __init__(self, err_msg="", desc='', loc='', method='', driver=None, **kwargs):
        self.msg = err_msg
        self.desc = desc
        self.loc = loc
        self.method = method
        self.kwargs = kwargs
        self.driver = driver
        self.path = ''
        self.screenshots()

    def __str__(self):
        _msg = self.msg
        _msg += f', describe: {self.desc}' if self.desc else ''
        _msg += f', method: {self.method}' if self.method else ''
        _msg += f', loc: {self.loc}' if self.loc else ''
        _msg += f', screenshots: {self.path}' if self.path else ''
        return _msg

    def screenshots(self):
        from P_EO.core.driver import Driver

        if Default.SCREENSHOTS and self.driver:
            if not isinstance(self.driver, Driver):
                self.driver = Driver(self.driver)
            os.makedirs(Default.SCREENSHOTS_PATH, exist_ok=True)
            self.path = os.path.join(Default.SCREENSHOTS_PATH, f'{get_time()}_{self.desc}.png')
            self.driver.screenshots(self.path)


class ElementNotFoundError(PEOBaseError):
    def __init__(self, desc, loc, method, driver=None):
        super().__init__(desc=desc, loc=loc, method=method, driver=driver)
        self.msg = '元素未找到'


class ElementFindTimeoutError(PEOBaseError):
    def __init__(self, desc, loc, method, driver=None):
        super().__init__(desc=desc, loc=loc, method=method, driver=driver)
        self.msg = '元素查找超时'


class ElementNotDisplayedError(PEOBaseError):
    def __init__(self, desc, loc, method, driver=None):
        super().__init__(desc=desc, loc=loc, method=method, driver=driver)
        self.msg = '元素存在但是不可见'


class ElementNotEnableError(PEOBaseError):
    def __init__(self, desc, loc, method, driver=None):
        super().__init__(desc=desc, loc=loc, method=method, driver=driver)
        self.msg = '元素存在但是不可交互'


class ElementInputError(PEOBaseError):
    def __init__(self, desc, loc, method, send_value, tag_value, driver=None):
        super().__init__(desc=desc, loc=loc, method=method, send_value=send_value, tag_value=tag_value, driver=driver)
        self.msg = '元素输入内容错误'
